<?php

namespace Drupal\fundraise_up_api\Form\Events;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class GetEventsForm.
 *
 * @package Drupal\fundraise_up_api\Form\Events
 */
class GetEventsForm extends FormBase {

  /**
   * List of available Event types from FundraiseUp documentation.
   *
   * @see https://fundraiseup.com/docs/rest-api-resources/#events.
   *
   * @var array
   */
  protected const EVENT_TYPES = [
    'donation.created' => 'Donation created',
    'donation.success' => 'Donation successful',
    'donation.pending' => 'Donation pending',
    'donation.failed' => 'Donation failed',
    'donation.updated' => 'Donation updated',
    'donation.refunded' => 'Donation refunded',
    'donation.success_early_fraud_warning' => 'Donation early fraud warning',
    'donation.pending_ach_verification_required' => 'Donation pending (verification required)',
    'donation.pending_incomplete' => "Donation pending (waiting for the payment provider's response)",
    'donation.pending_scheduled' => 'Donation scheduled',
    'donation.pending_waiting_for_transfer' => 'Donation pending (waiting for transfer)',
    'donation.pending_paypal_action_required' => 'Donation pending (PayPal action required)',
    'donation.pending_retrying' => 'Donation retrying',
    'donation.personal_data_deleted' => 'Donation personal information deleted',
    'donation.success_dispute_created' => 'Donation dispute created',
    'donation.success_dispute_lost' => 'Donation dispute lost',
    'donation.success_dispute_won' => 'Donation dispute won',
    'donation.success_dispute_in_review' => 'Donation dispute in review',
    'recurring_plan.activated' => 'Recurring plan activated',
    'recurring_plan.scheduled' => 'Recurring plan scheduled',
    'recurring_plan.failed' => 'Recurring plan failed',
    'recurring_plan.completed' => 'Recurring plan completed',
    'recurring_plan.canceled' => 'Recurring plan canceled',
    'recurring_plan.terms_updated' => 'Recurring plan payment terms updated',
    'recurring_plan.paused' => 'Recurring plan paused',
    'recurring_plan.retrying' => 'Recurring plan retrying',
    'recurring_plan.details_updated' => 'Recurring plan supporter preferences updated',
    'recurring_plan.personal_data_deleted' => 'Recurring plan personal information deleted',
    'supporter.created' => 'Supporter created',
    'supporter.personal_data_deleted' => 'Supporter details deleted',
    'supporter.updated' => 'Supporter updated',
    'transaction_attempt.pending' => 'Transaction attempt pending',
    'transaction_attempt.success' => 'Transaction attempt successful',
    'transaction_attempt.failed' => 'Transaction attempt failed',
    'tribute.created' => 'Tribute created',
    'tribute.updated' => 'Tribute updated',
    'tribute.shared' => 'Tribute card sent',
  ];

  /**
   * Rest Api client.
   *
   * @var \Drupal\fundraise_up_api\Http\RestApiClient
   */
  protected $restApiClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->restApiClient = $container
      ->get('fundraise_up_api.manager')
      ->getRestApiClient();

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'fundraise_up_api_get_events_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['fieldset'] = [
      '#type' => 'fieldset',
      '#collapsible' => FALSE,
      '#title' => $this->t('Retrieve all events'),
      '#tree' => FALSE,
    ];

    if (!$this->restApiClient) {
      $form['fieldset']['error'] = [
        '#type' => 'markup',
        '#markup' => $this->t('Configure Fundraise Up API module before making requests.'),
      ];

      return $form;
    }

    $result = $form_state->get('result');

    if ($result) {
      $form['fieldset']['result'] = [
        '#type' => 'textarea',
        '#title' => $this->t('The result'),
        '#disabled' => TRUE,
        '#rows' => 20,
        '#default_value' => print_r($result, TRUE),
      ];
    }

    $form['fieldset']['events_records'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Number of records'),
      '#required' => TRUE,
      '#default_value' => 10,
      '#description' => $this->t('Specify the number of records per page, ranging from 1 to 100.'),
    ];

    $form['fieldset']['events_types'] = [
      '#type' => 'select',
      '#multiple' => TRUE,
      '#title' => $this->t('Event types'),
      '#required' => TRUE,
      '#options' => $this::EVENT_TYPES,
      '#size' => 10,
      '#weight' => 0,
      '#default_value' => !empty($form_state->get('events_types')) ? $form_state->get('events_types') : ['donation.updated'],
      '#description' => $this->t('Select up to 10 event types. The list will be filtered to include only events with a matching event type property. Passing more than 10 event types will result in an error.'),
    ];

    $form['fieldset']['events_ending_before'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Ending before'),
      '#description' => $this->t('Use an existing object ID to define your position in the list. For example, if you fetch a list of items and receive 100 results, you can use the ID of the first object in that list as the ending_before parameter in your subsequent request to retrieve the previous page of results.'),
    ];
    $form['fieldset']['events_starting_after'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Starting after'),
      '#description' => $this->t('Use an existing object ID to define your position in the list. For example, if you fetch a list of items and receive 100 results, you can use the ID of the last object in that list as the starting_after parameter in your subsequent request to retrieve the next page of results.'),
    ];

    $form['fieldset']['actions']['#type'] = 'actions';

    $form['fieldset']['actions']['submit_events'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
      '#button_type' => 'primary',
    ];

    // Prevents cache-serialize exception.
    // The problem can be investigated later.
    $form_state->disableCache();

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $events = $this->restApiClient->events();
    $types = array_keys($form_state->getValue('events_types'));
    $records = $form_state->getValue('events_records');
    $endingBefore = $form_state->getValue('events_ending_before');
    $startingAfter = $form_state->getValue('events_starting_after');

    try {
      $response = $events->getEvents($types, $records, $endingBefore, $startingAfter);
      $form_state->set('result', $response->getData());
    }
    catch (\Throwable $e) {
      $this->messenger()->addError($e->getMessage());
    }

    $form_state->setRebuild();
  }

}
